# Generated by roxygen2: do not edit by hand

export(GetExchangeRates)
export(continue_sequence)
export(date_bump_month)
export(duplicate_last_list_element)
export(get_exchange_rates_symbol)
export(runInvoice)
export(try_exchange_rates_direct_and_indirect)
importFrom(dplyr,bind_rows)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,rowwise)
importFrom(lubridate,day)
importFrom(lubridate,month)
importFrom(lubridate,year)
importFrom(quantmod,getFX)
importFrom(quantmod,getSymbols)
importFrom(rlang,":=")
importFrom(rlang,set_names)
importFrom(shiny,isTruthy)
importFrom(stringr,str_replace_all)
importFrom(tibble,rownames_to_column)
importFrom(utils,install.packages)
