library(rhino)
library(tinytex)
library(pillar)
library(rjson)
library(lubridate)
library(rmarkdown)
library(readr)
library(dplyr)
library(quantmod)
library(shinyAce)
library(fs)
library(jsonlite)
library(stringr)
